
(IN-PACKAGE "USER") 
(PROCLAIM
    '(FTYPE (FUNCTION (FIXNUM FIXNUM T) *) XLIB:QUERY-BEST-CURSOR
            XLIB:QUERY-BEST-STIPPLE XLIB:QUERY-BEST-TILE)) 
(PROCLAIM
    '(FTYPE (FUNCTION (FIXNUM FIXNUM T T FIXNUM T T) *)
            XLIB::IMAGE-SWAP-FUNCTION)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                ((VECTOR UNSIGNED-CHAR) FIXNUM T FIXNUM FIXNUM FIXNUM
                 FIXNUM FIXNUM FIXNUM T FIXNUM T T FIXNUM T T)
                *)
            XLIB::READ-PIXARRAY-INTERNAL XLIB::WRITE-PIXARRAY-INTERNAL)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                ((VECTOR UNSIGNED-CHAR) FIXNUM T FIXNUM FIXNUM FIXNUM
                 FIXNUM FIXNUM FIXNUM FIXNUM T T)
                *)
            XLIB::READ-PIXARRAY XLIB::WRITE-PIXARRAY
            XLIB::FAST-READ-PIXARRAY XLIB::FAST-WRITE-PIXARRAY)) 
(PROCLAIM '(FTYPE (FUNCTION (T INTEGER *) *) XLIB:KEYSYM->CHARACTER)) 
(PROCLAIM '(FTYPE (FUNCTION (T INTEGER T *) *) XLIB:STORE-COLOR)) 
(PROCLAIM '(FTYPE (FUNCTION (*) (VALUES T T)) XLIB::XINTERN)) 
(PROCLAIM
    '(FTYPE (FUNCTION ((VECTOR T)) *) XLIB::DEALLOCATE-GCONTEXT-STATE)) 
(PROCLAIM
    '(FTYPE (FUNCTION (FIXNUM T FIXNUM) FIXNUM) XLIB::ASET-CARD8
            XLIB::ASET-INT8 XLIB::ASET-CARD29 XLIB::ASET-INT16
            XLIB::ASET-CARD16 XLIB::ASET-INT32)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM FIXNUM) FIXNUM)
            XLIB:WARP-POINTER-RELATIVE XLIB:WARP-POINTER)) 
(PROCLAIM
    '(FTYPE (FUNCTION (FIXNUM FIXNUM T FIXNUM FIXNUM *) T)
            XLIB:WARP-POINTER-RELATIVE-IF-INSIDE)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM FIXNUM T T FIXNUM) FIXNUM)
            XLIB:TRANSLATE-DEFAULT)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) XLIB::BUFFER-BYTES) XLIB::BUFFER-OBUF8
            XLIB::REPLY-IBUF8)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) XLIB:CARD16) XLIB::FONT-INFO-DEFAULT-CHAR
            XLIB::IMAGE-X-BYTES-PER-LINE
            XLIB:DISPLAY-MAX-REQUEST-LENGTH
            XLIB:SCREEN-MAX-INSTALLED-MAPS
            XLIB:SCREEN-MIN-INSTALLED-MAPS
            XLIB:VISUAL-INFO-COLORMAP-ENTRIES XLIB:IMAGE-WIDTH
            XLIB:SCREEN-HEIGHT-IN-MILLIMETERS XLIB:SCREEN-WIDTH
            XLIB:SCREEN-HEIGHT XLIB:DISPLAY-PROTOCOL-MAJOR-VERSION
            XLIB:DISPLAY-PROTOCOL-MINOR-VERSION
            XLIB:SCREEN-WIDTH-IN-MILLIMETERS XLIB:IMAGE-HEIGHT
            XLIB::PENDING-COMMAND-SEQUENCE XLIB::FONT-INFO-MIN-CHAR
            XLIB::FONT-INFO-MAX-CHAR)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) XLIB:CARD8) XLIB::FONT-INFO-MIN-BYTE1
            XLIB::FONT-INFO-MAX-BYTE1 XLIB::FONT-INFO-MIN-BYTE2
            XLIB::FONT-INFO-MAX-BYTE2 XLIB::IMAGE-X-LEFT-PAD
            XLIB:DISPLAY-MAX-KEYCODE XLIB:IMAGE-DEPTH
            XLIB:VISUAL-INFO-BITS-PER-RGB XLIB:DISPLAY-MIN-KEYCODE)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) XLIB:CARD32) XLIB:WM-HINTS-FLAGS
            XLIB:DISPLAY-MOTION-BUFFER-SIZE
            XLIB:DISPLAY-RELEASE-NUMBER)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) FIXNUM) XLIB:UNMAP-SUBWINDOWS
            XLIB::FONT-REFERENCE-COUNT XLIB:CIRCULATE-WINDOW-UP
            XLIB::DISPLAY-SIZE XLIB::BUFFER-NEW-REQUEST-NUMBER
            XLIB::DISPLAY-BOFFSET XLIB:ADD-TO-SAVE-SET
            XLIB:UNMAP-WINDOW XLIB:MAP-WINDOW XLIB:NO-OPERATION
            XLIB:CIRCULATE-WINDOW-DOWN XLIB:GRAB-SERVER
            XLIB:SCREEN-ROOT-VISUAL XLIB:INSTALL-COLORMAP
            XLIB:ACTIVATE-SCREEN-SAVER XLIB:REMOVE-FROM-SAVE-SET
            XLIB:UNINSTALL-COLORMAP XLIB:DESTROY-SUBWINDOWS
            XLIB:DESTROY-WINDOW XLIB:MAP-SUBWINDOWS
            XLIB:RESET-SCREEN-SAVER XLIB:KILL-TEMPORARY-CLIENTS
            XLIB:UNGRAB-SERVER XLIB:DISPLAY-NSCREENS)) 
(PROCLAIM
    '(FTYPE (FUNCTION (FIXNUM) T) XLIB::INT32->CARD32
            XLIB::ALLOCATE-REPLY-BUFFER XLIB::CARD16->RGB-VAL
            XLIB::INT16->RADIANS XLIB:MAKE-STATE-KEYS)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (FUNCTION (XLIB:GCONTEXT T) T))
            XLIB::GCONTEXT-EXTENSION-SET-FUNCTION)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) XLIB:IMAGE-DEPTH) XLIB:SCREEN-ROOT-DEPTH
            XLIB:PIXMAP-FORMAT-DEPTH)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (FUNCTION (XLIB:GCONTEXT XLIB:GCONTEXT T) T))
            XLIB::GCONTEXT-EXTENSION-COPY-FUNCTION)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) XLIB:BITMAP-FORMAT)
            XLIB:DISPLAY-BITMAP-FORMAT)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) LIST) XLIB::DISPLAY-EXTENSION-ALIST
            XLIB::RESOURCE-DATABASE-TIGHT XLIB::RESOURCE-DATABASE-LOOSE
            XLIB::DISPLAY-MODIFIER-MAPPING
            XLIB::DISPLAY-KEYSYM-TRANSLATION XLIB:DISPLAY-ROOTS
            XLIB:SCREEN-PLIST XLIB:DISPLAY-REPORT-ASYNCHRONOUS-ERRORS
            XLIB:IMAGE-XY-BITMAP-LIST XLIB:FONT-PLIST XLIB:IMAGE-PLIST
            XLIB:GCONTEXT-PLIST XLIB:VISUAL-INFO-PLIST)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T)
                (MEMBER :STATIC-GRAY :STATIC-COLOR :TRUE-COLOR
                        :GRAY-SCALE :PSEUDO-COLOR :DIRECT-COLOR))
            XLIB:VISUAL-INFO-CLASS)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (MEMBER 8 16 32)) XLIB:BITMAP-FORMAT-PAD
            XLIB:PIXMAP-FORMAT-SCANLINE-PAD XLIB::IMAGE-X-UNIT
            XLIB::IMAGE-X-PAD XLIB:BITMAP-FORMAT-UNIT)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T INTEGER *) T) XLIB:UNDEFINE-KEYSYM
            XLIB:DEFINE-KEYSYM)) 
(PROCLAIM '(FTYPE (FUNCTION (T) XLIB:PIXARRAY) XLIB:IMAGE-Z-PIXARRAY)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) XLIB:PIXEL) XLIB:SCREEN-BLACK-PIXEL
            XLIB::STANDARD-COLORMAP-BASE-PIXEL
            XLIB:VISUAL-INFO-BLUE-MASK XLIB:VISUAL-INFO-GREEN-MASK
            XLIB:VISUAL-INFO-RED-MASK XLIB:SCREEN-WHITE-PIXEL)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) XLIB:RESOURCE-ID)
            XLIB::DISPLAY-RESOURCE-ID-COUNT XLIB:GCONTEXT-ID
            XLIB:COLORMAP-ID XLIB:CURSOR-ID
            XLIB:DISPLAY-RESOURCE-ID-MASK XLIB:DISPLAY-RESOURCE-ID-BASE
            XLIB:VISUAL-INFO-ID)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (MEMBER 1 4 8 16 24 32))
            XLIB::IMAGE-X-BITS-PER-PIXEL XLIB:IMAGE-Z-BITS-PER-PIXEL
            XLIB:PIXMAP-FORMAT-BITS-PER-PIXEL)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (MEMBER :NEVER :WHEN-MAPPED :ALWAYS))
            XLIB:SCREEN-BACKING-STORES)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) XLIB:MASK32) XLIB:SCREEN-EVENT-MASK-AT-OPEN)) 
(PROCLAIM
    '(FTYPE (FUNCTION ((VECTOR UNSIGNED-CHAR) FIXNUM T) T)
            XLIB::READ-BITVECTOR256)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T INTEGER BIT-VECTOR) T) XLIB:KEYSYM-IN-MAP-P)) 
(PROCLAIM '(FTYPE (FUNCTION (T) SEQUENCE) XLIB:DISPLAY-PIXMAP-FORMATS)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) VECTOR) XLIB::DISPLAY-EVENT-EXTENSIONS)) 
(PROCLAIM '(FTYPE (FUNCTION (T) SYMBOL) XLIB::GCONTEXT-EXTENSION-NAME)) 
(PROCLAIM '(FTYPE (FUNCTION (T INTEGER T) T) XLIB::SAVE-ID)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (VALUES T T)) XLIB::KINTERN XLIB::GETIFY
            XLIB::CANONICALIZE-EVENT-NAME)) 
(PROCLAIM
    '(FTYPE (FUNCTION ((VECTOR UNSIGNED-CHAR) FIXNUM FIXNUM T *) T)
            XLIB::READ-SEQUENCE-STRING)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                ((VECTOR UNSIGNED-CHAR) FIXNUM (ARRAY UNSIGNED-SHORT) T
                 T FIXNUM FIXNUM FIXNUM FIXNUM)
                T)
            XLIB::READ-PIXARRAY-16)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                ((VECTOR UNSIGNED-CHAR) FIXNUM (ARRAY FIXNUM) T T
                 FIXNUM FIXNUM FIXNUM FIXNUM)
                T)
            XLIB::READ-PIXARRAY-24)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                ((VECTOR UNSIGNED-CHAR) FIXNUM (ARRAY T) T T FIXNUM
                 FIXNUM FIXNUM FIXNUM)
                T)
            XLIB::READ-PIXARRAY-32)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                ((VECTOR UNSIGNED-CHAR) FIXNUM FIXNUM T FIXNUM FIXNUM
                 FIXNUM FIXNUM FIXNUM FIXNUM T T FIXNUM)
                T)
            XLIB::READ-XY-FORMAT-IMAGE-X)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (MEMBER :BITMAP :XY-PIXMAP :Z-PIXMAP))
            XLIB::IMAGE-X-FORMAT)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                ((VECTOR UNSIGNED-CHAR) FIXNUM FIXNUM T FIXNUM FIXNUM
                 FIXNUM FIXNUM FIXNUM T T FIXNUM FIXNUM)
                T)
            XLIB::READ-Z-FORMAT-IMAGE-X)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                ((VECTOR UNSIGNED-CHAR) FIXNUM FIXNUM T FIXNUM FIXNUM
                 FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM T T)
                T)
            XLIB::READ-IMAGE-XY XLIB::READ-IMAGE-Z)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                ((VECTOR UNSIGNED-CHAR) FIXNUM (ARRAY BIT) FIXNUM
                 FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM)
                T)
            XLIB::READ-PIXARRAY-1 XLIB::WRITE-PIXARRAY-1)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                ((VECTOR UNSIGNED-CHAR) FIXNUM (ARRAY UNSIGNED-CHAR)
                 FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM)
                T)
            XLIB::READ-PIXARRAY-4 XLIB::READ-PIXARRAY-8
            XLIB::WRITE-PIXARRAY-4 XLIB::WRITE-PIXARRAY-8)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                ((VECTOR UNSIGNED-CHAR) FIXNUM (ARRAY UNSIGNED-SHORT)
                 FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM)
                T)
            XLIB::WRITE-PIXARRAY-16)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                ((VECTOR UNSIGNED-CHAR) FIXNUM (ARRAY FIXNUM) FIXNUM
                 FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM)
                T)
            XLIB::WRITE-PIXARRAY-24)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                ((VECTOR UNSIGNED-CHAR) FIXNUM (ARRAY T) FIXNUM FIXNUM
                 FIXNUM FIXNUM FIXNUM FIXNUM)
                T)
            XLIB::WRITE-PIXARRAY-32)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T INTEGER INTEGER) T) XLIB:DEFINE-KEYSYM-SET)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T (VECTOR T)) FIXNUM)
            XLIB::PUT-WINDOW-ATTRIBUTE-CHANGES
            XLIB::PUT-DRAWABLE-GEOMETRY-CHANGES)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) HASH-TABLE) XLIB::DISPLAY-ATOM-ID-MAP
            XLIB::DISPLAY-ATOM-CACHE XLIB::DISPLAY-RESOURCE-ID-MAP)) 
(PROCLAIM '(FTYPE (FUNCTION (T *) (VALUES T T)) XLIB::PUTIFY)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T) FIXNUM) XLIB:WITHDRAW-WINDOW
            XLIB::BYTE-REF16 XLIB:DELETE-PROPERTY XLIB:ICONIFY-WINDOW)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM) T) XLIB::LOOKUP-XATOM
            XLIB::BUFFER-ENSURE-SIZE XLIB::BUFFER-PAD-REQUEST
            XLIB::GET-ERROR-KEY XLIB::SET-GCONTEXT-FUNCTION
            XLIB::SET-GCONTEXT-LINE-WIDTH XLIB::SET-GCONTEXT-TS-X
            XLIB::SET-GCONTEXT-TS-Y XLIB::SET-GCONTEXT-CLIP-X
            XLIB::SET-GCONTEXT-CLIP-Y XLIB::SET-GCONTEXT-DASH-OFFSET
            XLIB:VISUAL-INFO)) 
(PROCLAIM '(FTYPE (FUNCTION ((VECTOR T) T) T) XLIB::DECODE-WM-HINTS)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T) (VALUES T T)) XLIB::LOOKUP-RESOURCE-ID
            XLIB::ATOM-ID)) 
(PROCLAIM '(FTYPE (FUNCTION (INTEGER) *) XLIB::CARD32->INT32)) 
(PROCLAIM
    '(FTYPE (FUNCTION (FIXNUM) FIXNUM) XLIB::INT16->CARD16
            XLIB::CARD8->INT8 XLIB::INT8->CARD8 XLIB::CARD16->INT16)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) XLIB:ARRAY-INDEX) XLIB::BUFFER-BOFFSET
            XLIB::REPLY-SIZE XLIB::REPLY-DATA-SIZE XLIB::BUFFER-SIZE)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (ARRAY XLIB:CARD8 (*))) XLIB::IMAGE-X-DATA)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) XLIB:DRAW-DIRECTION)
            XLIB::FONT-INFO-DIRECTION)) 
(PROCLAIM '(FTYPE (FUNCTION (T) INTEGER) XLIB:DISPLAY-DISPLAY)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) XLIB:INT16) XLIB::FONT-INFO-ASCENT
            XLIB::FONT-INFO-DESCENT)) 
(PROCLAIM
    '(FTYPE (FUNCTION (INTEGER) T) XLIB:MAKE-EVENT-KEYS XLIB:KEYSYM-SET
            XLIB::KEYSYM-DOWNCASE XLIB::KEYSYM-UPPERCASE-ALPHABETIC-P)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL (NOT NULL))) XLIB::BUFFER-DEAD
            XLIB::BUFFER-FLUSH-INHIBIT)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL STREAM)) XLIB::BUFFER-INPUT-STREAM
            XLIB::BUFFER-OUTPUT-STREAM)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL XLIB:COLOR))
            XLIB::STANDARD-COLORMAP-MAX-COLOR
            XLIB::STANDARD-COLORMAP-MULT-COLOR)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL XLIB:DISPLAY)) XLIB:CURSOR-DISPLAY
            XLIB:FONT-DISPLAY XLIB:GCONTEXT-DISPLAY
            XLIB:COLORMAP-DISPLAY XLIB:VISUAL-INFO-DISPLAY)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T)
                (OR (MEMBER NIL :RELEASE-BY-FREEING-COLORMAP)
                    XLIB:DRAWABLE XLIB:GCONTEXT XLIB:CURSOR
                    XLIB:COLORMAP XLIB:FONT))
            XLIB::STANDARD-COLORMAP-KILL)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL XLIB:DRAWABLE))
            XLIB::GCONTEXT-DRAWABLE)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL XLIB::PENDING-COMMAND))
            XLIB::PENDING-COMMAND-NEXT)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL XLIB:GCONTEXT)) XLIB::GCONTEXT-NEXT)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL VECTOR)) XLIB::FONT-INFO-MIN-BOUNDS
            XLIB::FONT-INFO-MAX-BOUNDS)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL XLIB:RESOURCE-ID))
            XLIB:WM-HINTS-WINDOW-GROUP XLIB::FONT-ID-INTERNAL)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL XLIB:INT16)) XLIB:WM-SIZE-HINTS-Y
            XLIB:WM-SIZE-HINTS-X)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL XLIB::FONT-INFO))
            XLIB::FONT-FONT-INFO-INTERNAL)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL XLIB:CARD16))
            XLIB:WM-SIZE-HINTS-BASE-WIDTH XLIB:WM-SIZE-HINTS-MAX-WIDTH
            XLIB:WM-SIZE-HINTS-MIN-WIDTH XLIB:WM-SIZE-HINTS-WIDTH
            XLIB:WM-SIZE-HINTS-HEIGHT-INC XLIB:WM-SIZE-HINTS-WIDTH-INC
            XLIB:WM-HINTS-ICON-Y XLIB:WM-SIZE-HINTS-BASE-HEIGHT
            XLIB:WM-SIZE-HINTS-HEIGHT XLIB:WM-SIZE-HINTS-MAX-HEIGHT
            XLIB:WM-SIZE-HINTS-MIN-HEIGHT XLIB:WM-HINTS-ICON-X)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL (SIMPLE-ARRAY XLIB:INT16 (*))))
            XLIB::FONT-CHAR-INFOS-INTERNAL)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL XLIB:COLORMAP))
            XLIB:SCREEN-DEFAULT-COLORMAP
            XLIB::STANDARD-COLORMAP-COLORMAP)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (UNSIGNED-BYTE 16))
            XLIB::BUFFER-REQUEST-NUMBER)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) UNSIGNED-SHORT) XLIB::IMAGE-X-WIDTH
            XLIB::IMAGE-X-HEIGHT XLIB::IMAGE-XY-WIDTH
            XLIB::DISPLAY-REQUEST-NUMBER XLIB::IMAGE-XY-HEIGHT
            XLIB::IMAGE-Z-WIDTH XLIB::IMAGE-Z-HEIGHT)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL (MEMBER :OFF :ON)))
            XLIB:WM-HINTS-INPUT)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL XLIB:WINDOW))
            XLIB:WM-HINTS-ICON-WINDOW XLIB:SCREEN-ROOT)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) UNSIGNED-CHAR) XLIB::IMAGE-X-DEPTH
            XLIB::IMAGE-XY-DEPTH XLIB::IMAGE-Z-DEPTH)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL XLIB:VISUAL-INFO))
            XLIB:SCREEN-ROOT-VISUAL-INFO XLIB:COLORMAP-VISUAL-INFO
            XLIB::STANDARD-COLORMAP-VISUAL)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL XLIB::REPLY-BUFFER))
            XLIB::DISPLAY-NEW-EVENTS XLIB::REPLY-NEXT
            XLIB::PENDING-COMMAND-REPLY-BUFFER
            XLIB::DISPLAY-EVENT-QUEUE-TAIL
            XLIB::DISPLAY-EVENT-QUEUE-HEAD)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL XLIB:CARD29))
            XLIB::DISPLAY-LAST-WIDTH)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL XLIB:ARRAY-INDEX))
            XLIB::BUFFER-LAST-REQUEST
            XLIB::BUFFER-LAST-FLUSHED-REQUEST)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL (ARRAY * (* *))))
            XLIB::DISPLAY-KEYSYM-MAPPING)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL XLIB:PIXMAP))
            XLIB:WM-HINTS-ICON-MASK XLIB:WM-HINTS-ICON-PIXMAP)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL XLIB:WIN-GRAVITY))
            XLIB:WM-SIZE-HINTS-WIN-GRAVITY)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) (OR NULL NUMBER))
            XLIB:WM-SIZE-HINTS-MAX-ASPECT
            XLIB:WM-SIZE-HINTS-MIN-ASPECT)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T)
                (OR NULL
                    (MEMBER :DONT-CARE :NORMAL :ZOOM :ICONIC :INACTIVE)))
            XLIB:WM-HINTS-INITIAL-STATE)) 
(PROCLAIM '(FTYPE (FUNCTION (T) (OR NULL STRING)) XLIB:FONT-NAME)) 
(PROCLAIM '(FTYPE (FUNCTION (T INTEGER) *) XLIB:KEYSYM->KEYCODES)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM) FIXNUM) XLIB:KILL-CLIENT
            XLIB::AREF-CARD8 XLIB::AREF-INT8 XLIB::AREF-CARD29
            XLIB::AREF-INT16 XLIB::AREF-CARD16
            XLIB::GET-INTERNAL-EVENT-CODE XLIB::AREF-INT32)) 
(PROCLAIM '(FTYPE (FUNCTION (FIXNUM T) (VALUES T T)) XLIB::ID-ATOM)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T INTEGER) T) XLIB::SET-GCONTEXT-PLANE-MASK
            XLIB::SET-GCONTEXT-FOREGROUND
            XLIB::SET-GCONTEXT-BACKGROUND)) 
(PROCLAIM
    '(FTYPE (FUNCTION ((VECTOR UNSIGNED-CHAR) FIXNUM) T)
            XLIB::AREF-CARD32 XLIB::BYTE-REF32)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T (VECTOR UNSIGNED-CHAR) FIXNUM FIXNUM T) T)
            XLIB::BUFFER-READ-DEFAULT)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T STRING FIXNUM FIXNUM) *)
            XLIB::TEXT-EXTENTS-SERVER XLIB::TEXT-WIDTH-SERVER)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T T) *) XLIB::UNMAP-NOTIFY-EVENT-GET
            XLIB::MAP-NOTIFY-EVENT-GET XLIB::MAP-REQUEST-EVENT-GET
            XLIB::REPARENT-NOTIFY-EVENT-GET
            XLIB::MOTION-NOTIFY-EVENT-GET
            XLIB::WITH-EVENT-QUEUE-FUNCTION
            XLIB::CONFIGURE-NOTIFY-EVENT-GET
            XLIB::CONFIGURE-REQUEST-EVENT-GET
            XLIB::GRAVITY-NOTIFY-EVENT-GET
            XLIB::RESIZE-REQUEST-EVENT-GET
            XLIB::CIRCULATE-NOTIFY-EVENT-GET
            XLIB::CIRCULATE-REQUEST-EVENT-GET
            XLIB::PROPERTY-NOTIFY-EVENT-GET
            XLIB::SELECTION-CLEAR-EVENT-GET
            XLIB::ENTER-NOTIFY-EVENT-GET
            XLIB::SELECTION-REQUEST-EVENT-GET XLIB::KEY-PRESS-EVENT-GET
            XLIB::FOCUS-IN-EVENT-GET XLIB::SELECTION-NOTIFY-EVENT-GET
            XLIB::KEYMAP-NOTIFY-EVENT-GET
            XLIB::COLORMAP-NOTIFY-EVENT-GET XLIB::WAIT-FOR-EVENT
            XLIB::EXPOSURE-EVENT-GET XLIB::CLIENT-MESSAGE-EVENT-GET
            XLIB::WITH-BUFFER-FUNCTION XLIB::MAPPING-NOTIFY-EVENT-GET
            XLIB::WITH-BUFFER-REQUEST-FUNCTION-NOLOCK
            XLIB::WITH-BUFFER-REQUEST-FUNCTION XLIB:SET-WM-CLASS
            XLIB::GRAPHICS-EXPOSURE-EVENT-GET
            XLIB::NO-EXPOSURE-EVENT-GET
            XLIB::VISIBILITY-NOTIFY-EVENT-GET
            XLIB::CREATE-NOTIFY-EVENT-GET
            XLIB::DESTROY-NOTIFY-EVENT-GET)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T T T) *) XLIB::EVENT-LOOP-STEP-BEFORE
            XLIB::WITH-BUFFER-REQUEST-AND-REPLY-FUNCTION)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T FIXNUM FIXNUM FIXNUM FIXNUM *) *)
            XLIB:DRAW-LINE XLIB:DRAW-RECTANGLE)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T *) *) XLIB::COPY-GCONTEXT-LOCAL-STATE
            XLIB::MAKE-BUFFER XLIB:MAP-RESOURCE XLIB:TEXT-WIDTH
            XLIB:WM-RESOURCES XLIB:GRAB-POINTER XLIB:QUERY-COLORS
            XLIB:GET-PROPERTY XLIB:WRITE-BITMAP-FILE XLIB:STORE-COLORS
            XLIB:READ-RESOURCES XLIB:TEXT-EXTENTS XLIB:QUEUE-EVENT)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T FIXNUM FIXNUM T *) *) XLIB:DRAW-GLYPH
            XLIB:DRAW-IMAGE-GLYPHS XLIB:DRAW-GLYPHS
            XLIB:DRAW-IMAGE-GLYPH)) 
(PROCLAIM '(FTYPE (FUNCTION (T T FIXNUM FIXNUM) *) XLIB:DRAW-POINT)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T T *) *) XLIB::GET-PUT-ITEMS
            XLIB::REPORT-ERROR XLIB:DRAW-ARCS)) 
(PROCLAIM '(FTYPE (FUNCTION (T STRING *) *) XLIB:LIST-FONT-NAMES)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T FIXNUM FIXNUM FIXNUM FIXNUM T T *) *)
            XLIB:DRAW-ARC)) 
(PROCLAIM '(FTYPE (FUNCTION (T T T T *) *) XLIB::READ-INPUT)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T INTEGER INTEGER T) *)
            XLIB::TEXT-EXTENTS-LOCAL)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T FIXNUM FIXNUM T FIXNUM T T T) *)
            XLIB::DRAW-GLYPHS8 XLIB::DRAW-GLYPHS16
            XLIB::DRAW-IMAGE-GLYPHS8 XLIB::DRAW-IMAGE-GLYPHS16)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) XLIB:BOOLEAN) XLIB:SCREEN-SAVE-UNDERS-P
            XLIB::WM-SIZE-HINTS-PROGRAM-SPECIFIED-POSITION-P
            XLIB::WM-SIZE-HINTS-PROGRAM-SPECIFIED-SIZE-P
            XLIB::FONT-INFO-ALL-CHARS-EXIST-P XLIB::FONT-LOCAL-ONLY-P
            XLIB::IMAGE-X-BIT-LSB-FIRST-P
            XLIB::IMAGE-X-BYTE-LSB-FIRST-P
            XLIB:DISPLAY-IMAGE-LSB-FIRST-P
            XLIB:WM-SIZE-HINTS-USER-SPECIFIED-SIZE-P
            XLIB:BITMAP-FORMAT-LSB-FIRST-P
            XLIB:WM-SIZE-HINTS-USER-SPECIFIED-POSITION-P
            XLIB:GCONTEXT-CACHE-P)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) CONS) XLIB::DISPLAY-CURRENT-EVENT-SYMBOL)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) XLIB::FONT-PROPS) XLIB::FONT-INFO-PROPERTIES)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) XLIB::GCONTEXT-STATE)
            XLIB::GCONTEXT-SERVER-STATE XLIB::GCONTEXT-LOCAL-STATE)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T CHARACTER BIT-VECTOR) T)
            XLIB:CHARACTER-IN-MAP-P)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) XLIB:RGB-VAL) XLIB:COLOR-RED XLIB:COLOR-BLUE
            XLIB:COLOR-GREEN)) 
(PROCLAIM '(FTYPE (FUNCTION (T STRING *) T) XLIB:LIST-FONTS)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T T) T) XLIB:ADD-RESOURCE
            XLIB::SET-GCONTEXT-CLIP-MASK
            XLIB::GRAPHICS-EXPOSURE-EVENT-GET-MACRO
            XLIB::NO-EXPOSURE-EVENT-GET-MACRO
            XLIB::PRINT-RESOURCE-DATABASE XLIB::PRINT-GCONTEXT
            XLIB::VISIBILITY-NOTIFY-EVENT-GET-MACRO
            XLIB::CREATE-NOTIFY-EVENT-GET-MACRO
            XLIB::DESTROY-NOTIFY-EVENT-GET-MACRO
            XLIB::UNMAP-NOTIFY-EVENT-GET-MACRO
            XLIB::MAP-NOTIFY-EVENT-GET-MACRO XLIB::MASK-PUT
            XLIB::PRINT-IMAGE XLIB::MASK-GET
            XLIB::MAP-REQUEST-EVENT-GET-MACRO
            XLIB::REPARENT-NOTIFY-EVENT-GET-MACRO
            XLIB::SET-WINDOW-PRIORITY XLIB::SET-EVENT-HANDLER
            XLIB::PRINT-DRAWABLE XLIB::CONFIGURE-NOTIFY-EVENT-GET-MACRO
            XLIB::PRINT-COLOR XLIB::CONFIGURE-REQUEST-EVENT-GET-MACRO
            XLIB::MAKE-COLOR-INTERNAL
            XLIB::GRAVITY-NOTIFY-EVENT-GET-MACRO
            XLIB::RESIZE-REQUEST-EVENT-GET-MACRO
            XLIB::CIRCULATE-NOTIFY-EVENT-GET-MACRO
            XLIB::CIRCULATE-REQUEST-EVENT-GET-MACRO
            XLIB::KEY-PRESS-EVENT-GET-MACRO
            XLIB::PROPERTY-NOTIFY-EVENT-GET-MACRO
            XLIB:GET-SEARCH-RESOURCE
            XLIB::SELECTION-CLEAR-EVENT-GET-MACRO
            XLIB::ENTER-NOTIFY-EVENT-GET-MACRO
            XLIB::SELECTION-REQUEST-EVENT-GET-MACRO
            XLIB::FOCUS-IN-EVENT-GET-MACRO
            XLIB::SELECTION-NOTIFY-EVENT-GET-MACRO
            XLIB::MOTION-NOTIFY-EVENT-GET-MACRO XLIB::MAKE-ERROR
            XLIB::KEYMAP-NOTIFY-EVENT-GET-MACRO
            XLIB::COLORMAP-NOTIFY-EVENT-GET-MACRO
            XLIB::EXPOSURE-EVENT-GET-MACRO
            XLIB::CLIENT-MESSAGE-EVENT-GET-MACRO
            XLIB::MAPPING-NOTIFY-EVENT-GET-MACRO
            XLIB::UNIVERSAL-EVENT-GET-MACRO XLIB::PRINT-VISUAL-INFO
            XLIB::PRINT-COLORMAP XLIB::PRINT-CURSOR XLIB::PRINT-DISPLAY
            XLIB::PRINT-SCREEN XLIB::OPEN-X-STREAM
            XLIB::SET-STRING-PROPERTY XLIB::PRINT-FONT
            XLIB:GET-SEARCH-TABLE)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T T *) T) XLIB:PUT-RAW-IMAGE
            XLIB::DRAW-ARCS-LIST XLIB::DRAW-ARCS-VECTOR
            XLIB:SET-INPUT-FOCUS XLIB:DRAW-POINTS
            XLIB:SET-SELECTION-OWNER XLIB:GRAB-BUTTON
            XLIB::TRACE-ERROR-PRINT XLIB::FIND-TRACE XLIB:DRAW-LINES
            XLIB:DRAW-RECTANGLES XLIB:SEND-EVENT XLIB:PUT-IMAGE
            XLIB:CONVERT-SELECTION)) 
(PROCLAIM '(FTYPE (FUNCTION (T T INTEGER *) T) XLIB:DESCRIBE-EVENT)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T *) T) XLIB:FREE-COLORS
            XLIB:CHANGE-KEYBOARD-MAPPING XLIB:UNGRAB-BUTTON
            XLIB:CHANGE-ACTIVE-POINTER-GRAB XLIB::X-CERROR
            XLIB::X-TYPE-ERROR XLIB::SET-CUT-BUFFER XLIB::PROCESS-BLOCK
            XLIB:GRAB-KEY XLIB:ROTATE-PROPERTIES
            XLIB:DEFAULT-ERROR-HANDLER XLIB:COPY-GCONTEXT-COMPONENTS
            XLIB:REMOVE-ACCESS-HOST XLIB:SET-WM-RESOURCES
            XLIB:ALLOW-EVENTS XLIB:DECODE-CORE-ERROR XLIB:IMAGE-PIXMAP
            XLIB:WRITE-RESOURCES XLIB:ADD-ACCESS-HOST XLIB:UNGRAB-KEY
            XLIB:CREATE-COLORMAP)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T T T FIXNUM *) T) XLIB:CHANGE-PROPERTY)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T FIXNUM FIXNUM) T) XLIB::BUFFER-WRITE
            XLIB:MAPPING-NOTIFY)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T T T T *) T) XLIB::EVENT-LOOP-STEP-AFTER)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T FIXNUM FIXNUM *) T) XLIB::BUFFER-INPUT)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM T T T) T)
            XLIB::WRITE-Z-FORMAT-IMAGE-X)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T T T T) T)
            XLIB::PRINT-UNREADABLE-OBJECT-FUNCTION
            XLIB::TRACE-MORE-INFO XLIB:GET-RESOURCE)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                (T T FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM T T) T)
            XLIB::WRITE-XY-FORMAT-IMAGE-X XLIB::WRITE-IMAGE-XY
            XLIB::WRITE-IMAGE-Z)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM) T)
            XLIB::FAST-COPY-PIXARRAY)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T T T) T) XLIB::PRINT-VALUE
            XLIB::ADD-GCONTEXT-EXTENSION XLIB::GET-ENTRY
            XLIB::GET-ENTRY-LOOKUP XLIB::GET-TABLES
            XLIB::GET-TABLES-LOOKUP XLIB::TRACE-READ-HOOK
            XLIB::TRACE-WRITE-HOOK)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T FIXNUM *) T) XLIB::READ-SEQUENCE-CHAR
            XLIB::READ-SEQUENCE-CARD8 XLIB::READ-SEQUENCE-INT8
            XLIB::READ-SEQUENCE-CARD16 XLIB::READ-SEQUENCE-INT16
            XLIB::READ-SEQUENCE-CARD32 XLIB::READ-SEQUENCE-INT32)) 
(PROCLAIM '(FTYPE (FUNCTION ((VECTOR T) INTEGER) T) XLIB::DECODE-MASK)) 
(PROCLAIM '(FTYPE (FUNCTION NIL *) XLIB::ALLOCATE-GCONTEXT-STATE)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                ((VECTOR UNSIGNED-CHAR) (VECTOR UNSIGNED-CHAR) FIXNUM
                 FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM
                 FIXNUM T T FIXNUM T T)
                *)
            XLIB::WRITE-XY-FORMAT-IMAGE-X-DATA)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                ((VECTOR UNSIGNED-CHAR) (VECTOR UNSIGNED-CHAR) FIXNUM
                 FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM
                 FIXNUM FIXNUM T T FIXNUM T T)
                *)
            XLIB::WRITE-Z-FORMAT-IMAGE-X-DATA)) 
(PROCLAIM
    '(FTYPE (FUNCTION NIL T) XLIB:MAKE-RESOURCE-DATABASE
            XLIB::ALLOCATE-TEMP-GCONTEXT XLIB::ALLOCATE-CONTEXT
            XLIB::DEFAULT-RESOURCES-PATHNAME XLIB::DECLARE-BUFMAC
            XLIB::RESOURCES-PATHNAME XLIB::DECLARE-BUFFUN
            XLIB::GC-CLEANUP XLIB::ALLOCATE-EVENT
            XLIB::CURRENT-PROCESS)) 
(PROCLAIM
    '(FTYPE (FUNCTION (*) *) XLIB::MAKE-DRAWABLE XLIB::MAKE-WINDOW
            XLIB::MAKE-PIXMAP)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                ((VECTOR UNSIGNED-CHAR) (VECTOR UNSIGNED-CHAR) FIXNUM
                 FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM T)
                T)
            XLIB::IMAGE-NOSWAP XLIB::IMAGE-SWAP-BITS-AND-TWO-BYTES
            XLIB::IMAGE-SWAP-BITS XLIB::IMAGE-SWAP-BITS-AND-FOUR-BYTES
            XLIB::IMAGE-SWAP-TWO-BYTES XLIB::IMAGE-SWAP-FOUR-BYTES
            XLIB::IMAGE-SWAP-WORDS XLIB::IMAGE-SWAP-BITS-AND-WORDS
            XLIB::IMAGE-SWAP-NIBBLES XLIB::IMAGE-SWAP-THREE-BYTES
            XLIB::IMAGE-SWAP-NIBBLES-LEFT)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                ((VECTOR UNSIGNED-CHAR) (VECTOR UNSIGNED-CHAR) FIXNUM
                 FIXNUM *)
                T)
            XLIB::BUFFER-REPLACE)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM FIXNUM T) *) XLIB::READ-REPLY-INPUT
            XLIB:TRANSLATE-COORDINATES)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM FIXNUM FIXNUM FIXNUM) *)
            XLIB::IMAGE-X->IMAGE-X)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM *) *) XLIB:ALLOC-COLOR-PLANES
            XLIB:ALLOC-COLOR-CELLS)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM FIXNUM *) *) XLIB:KEYCODE->CHARACTER)) 
(PROCLAIM
    '(FTYPE (FUNCTION (*) T) XLIB::MAKE-DISPLAY-INTERNAL
            XLIB::|__si::MAKE-BUFFER| XLIB::MAKE-FONT XLIB::MAKE-SCREEN
            XLIB::MAKE-FONT-INFO XLIB::MAKE-FONT-INTERNAL
            XLIB::MAKE-GCONTEXT-EXTENSION XLIB:CREATE-GLYPH-CURSOR
            XLIB:MAKE-WM-HINTS XLIB:BITMAP-IMAGE XLIB:MAKE-STATE-MASK
            XLIB:CREATE-PIXMAP XLIB:CREATE-IMAGE XLIB:MAKE-EVENT-MASK
            XLIB:MAKE-EVENT-HANDLERS XLIB:CREATE-WINDOW
            XLIB::MAKE-RESOURCE-DATABASE-INTERNAL XLIB::MAKE-GCONTEXT
            XLIB::|__si::MAKE-IMAGE| XLIB::MAKE-IMAGE-X
            XLIB::MAKE-IMAGE-XY XLIB:CREATE-GCONTEXT XLIB:DISPLAY-TRACE
            XLIB:MAKE-COLOR XLIB:CREATE-CURSOR XLIB::MAKE-IMAGE-Z
            XLIB::MAKE-REPLY-BUFFER-INTERNAL XLIB::|__si::MAKE-COLOR|
            XLIB::MAKE-BITMAP-FORMAT XLIB:MAKE-WM-SIZE-HINTS
            XLIB::MAKE-PIXMAP-FORMAT XLIB::MAKE-STANDARD-COLORMAP
            XLIB::MAKE-PENDING-COMMAND XLIB::MAKE-VISUAL-INFO
            XLIB::MAKE-COLORMAP XLIB::MAKE-CURSOR)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) *) XLIB:POINTER-POSITION XLIB:WM-NAME
            XLIB::WITH-STANDARD-IO-SYNTAX-FUNCTION
            XLIB::CLEANUP-STATE-ENTRY
            XLIB::GET-WINDOW-ATTRIBUTES-BUFFER
            XLIB::GET-DRAWABLE-GEOMETRY-BUFFER XLIB::EVENT-LOOP-SETUP
            XLIB::BUFFER-LISTEN XLIB:READ-BITMAP-FILE
            XLIB:DISPLAY-INVOKE-AFTER-FUNCTION XLIB:GCONTEXT-CLIP-MASK
            XLIB:INPUT-FOCUS XLIB:POINTER-CONTROL XLIB:GCONTEXT-STIPPLE
            XLIB:ACCESS-CONTROL XLIB:GET-WM-CLASS XLIB:TRANSIENT-FOR
            XLIB:WM-CLIENT-MACHINE XLIB:SCREEN-SAVER
            XLIB:KEYBOARD-CONTROL XLIB:RESOURCE-KEY
            XLIB:DISPLAY-PROTOCOL-VERSION XLIB:COLOR-RGB
            XLIB:GCONTEXT-TILE XLIB:DISPLAY-VENDOR XLIB:WM-ICON-NAME
            XLIB:DISPLAY-FINISH-OUTPUT XLIB:MODIFIER-MAPPING
            XLIB:QUERY-POINTER XLIB:DISPLAY-KEYCODE-RANGE
            XLIB:GLOBAL-POINTER-POSITION)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T)
                (XLIB:ALIST (XLIB:IMAGE-DEPTH XLIB::DEPTH)
                    ((LIST XLIB:VISUAL-INFO) XLIB::VISUALS)))
            XLIB:SCREEN-DEPTHS)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM FIXNUM FIXNUM FIXNUM) T)
            XLIB::IMAGE-XY->IMAGE-X XLIB::IMAGE-XY->IMAGE-XY
            XLIB::IMAGE-XY->IMAGE-Z XLIB::IMAGE-Z->IMAGE-X
            XLIB::IMAGE-Z->IMAGE-XY XLIB::IMAGE-Z->IMAGE-Z
            XLIB::XY-FORMAT-IMAGE-X->IMAGE-X
            XLIB::Z-FORMAT-IMAGE-X->IMAGE-X XLIB::IMAGE-X->IMAGE-XY
            XLIB::IMAGE-X->IMAGE-Z)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                (T FIXNUM (VECTOR UNSIGNED-SHORT) FIXNUM FIXNUM T) T)
            XLIB::WRITE-SIMPLE-ARRAY-CHAR2B-WITH-TRANSFORM
            XLIB::WRITE-SIMPLE-ARRAY-CARD16-WITH-TRANSFORM)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM (VECTOR UNSIGNED-SHORT) FIXNUM FIXNUM)
                T)
            XLIB::WRITE-SIMPLE-ARRAY-CHAR2B
            XLIB::READ-SIMPLE-ARRAY-CARD16
            XLIB::WRITE-SIMPLE-ARRAY-CARD16)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM (VECTOR UNSIGNED-CHAR) FIXNUM FIXNUM T)
                T)
            XLIB::WRITE-SIMPLE-ARRAY-CARD8-WITH-TRANSFORM)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T T T T) FIXNUM) XLIB:SET-SCREEN-SAVER
            XLIB::FILL-POLYGON)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM (VECTOR UNSIGNED-CHAR) FIXNUM FIXNUM)
                T)
            XLIB::READ-SIMPLE-ARRAY-CARD8
            XLIB::WRITE-SIMPLE-ARRAY-CARD8)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM (VECTOR SIGNED-SHORT) FIXNUM FIXNUM T)
                T)
            XLIB::WRITE-SIMPLE-ARRAY-INT16-WITH-TRANSFORM)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM (VECTOR SIGNED-SHORT) FIXNUM FIXNUM) T)
            XLIB::WRITE-SIMPLE-ARRAY-INT16)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T *) *) XLIB:EVENT-LISTEN XLIB:QUERY-TREE
            XLIB:GET-IMAGE XLIB:GRAB-KEYBOARD XLIB:QUERY-KEYMAP
            XLIB:FONT-PATH XLIB:INSTALLED-COLORMAPS XLIB:PROCESS-EVENT
            XLIB:MOTION-EVENTS XLIB:POINTER-MAPPING
            XLIB:SET-MODIFIER-MAPPING XLIB:CUT-BUFFER XLIB:ACCESS-HOSTS
            XLIB::CLIENT-MESSAGE-EVENT-PUT XLIB:GET-RAW-IMAGE
            XLIB:LIST-EXTENSIONS)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM (VECTOR T) FIXNUM FIXNUM T) T)
            XLIB::WRITE-SIMPLE-ARRAY-CARD32-WITH-TRANSFORM)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM (VECTOR T) FIXNUM FIXNUM) T)
            XLIB::READ-SIMPLE-ARRAY-CARD32
            XLIB::WRITE-SIMPLE-ARRAY-CARD32)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM T FIXNUM FIXNUM) T)
            XLIB::WRITE-LIST-CHAR2B XLIB::WRITE-VECTOR-CHAR2B
            XLIB::READ-LIST-CARD8 XLIB::READ-VECTOR-CARD8
            XLIB::READ-LIST-CARD16 XLIB::READ-VECTOR-CARD16
            XLIB::READ-LIST-CARD32 XLIB::READ-VECTOR-CARD32
            XLIB::WRITE-LIST-CARD8 XLIB::WRITE-VECTOR-CARD8
            XLIB::WRITE-LIST-CARD16 XLIB::WRITE-VECTOR-CARD16
            XLIB::WRITE-LIST-INT16 XLIB::WRITE-VECTOR-INT16
            XLIB::WRITE-LIST-CARD32 XLIB::WRITE-VECTOR-CARD32)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) T) XLIB::INITIALIZE-RESOURCE-ALLOCATOR
            XLIB:GCONTEXT-BACKGROUND XLIB::INITIALIZE-PREDEFINED-ATOMS
            XLIB::INITIALIZE-EXTENSIONS XLIB::MAKE-PROCESS-LOCK
            XLIB::MAKE-REPLY-BUFFER XLIB::GET-EVENT-CODE
            XLIB::BUFFER-FLUSH XLIB::DEALLOCATE-EVENT
            XLIB::RESOURCEALLOC XLIB::BUFFER-FORCE-OUTPUT-DEFAULT
            XLIB::IMAGE-X-PLIST XLIB:GCONTEXT-TS-Y
            XLIB::DISPLAY-KEYBOARD-MAPPING
            XLIB::GET-DISPLAY-MODIFIER-MAPPING XLIB:FREE-PIXMAP
            XLIB::PROCESS-WAKEUP XLIB:FREE-CURSOR
            XLIB:DISPLAY-XDEFAULTS XLIB:GCONTEXT-PLANE-MASK
            XLIB::DISPLAY-PERFORMANCE-INFO XLIB:WM-NORMAL-HINTS
            XLIB::DISPLAY-TRACE-HISTORY XLIB::DISPLAY-INPUT-IN-PROGRESS
            XLIB::RGB-VAL->CARD16 XLIB::DISPLAY-PENDING-COMMANDS
            XLIB:GCONTEXT-SUBWINDOW-MODE
            XLIB::DISPLAY-ASYNCHRONOUS-ERRORS
            XLIB::DISPLAY-EVENT-PROCESS XLIB::RADIANS->INT16
            XLIB::BUFFER-LISTEN-DEFAULT XLIB:GCONTEXT-FILL-RULE
            XLIB:WM-HINTS XLIB:DISPLAY-XID XLIB:DISCARD-CURRENT-EVENT
            XLIB:GCONTEXT-FILL-STYLE XLIB:GCONTEXT-LINE-STYLE
            XLIB::FONT-CHAR-INFOS XLIB:CLOSE-DOWN-MODE
            XLIB::ENCODE-EVENT-MASK XLIB::MAKE-INDEX-TYPED
            XLIB:COLORMAP-P XLIB::ENCODE-DEVICE-EVENT-MASK
            XLIB::DEALLOCATE-TEMP-GCONTEXT XLIB::ENCODE-MODIFIER-MASK
            XLIB::GCONTEXT-EXTENSION-DEFAULT XLIB::ENCODE-STATE-MASK
            XLIB::ENCODE-POINTER-EVENT-MASK XLIB:GCONTEXT-LINE-WIDTH
            XLIB::BIND-IO-HOOKS XLIB::UNBIND-IO-HOOKS
            XLIB:GCONTEXT-CLIP-Y XLIB::ENCODE-WM-HINTS
            XLIB::ENCODE-WM-SIZE-HINTS XLIB:DISPLAY-ERROR-HANDLER
            XLIB::DISPLAY-LOCK XLIB:GCONTEXT-DASH-OFFSET
            XLIB::EVENT-NUMBER XLIB:WM-COLORMAP-WINDOWS
            XLIB::DISPLAY-OUTPUT-STREAM XLIB:DRAWABLE-BORDER-WIDTH
            XLIB:GCONTEXT-EXPOSURES XLIB::DISPLAY-LAST-REQUEST
            XLIB:WM-COMMAND XLIB:WM-HINTS-P XLIB:WM-SIZE-HINTS-P
            XLIB:FORCE-GCONTEXT-CHANGES XLIB:WM-PROTOCOLS
            XLIB:DISPLAY-HOST XLIB:FREE-GCONTEXT
            XLIB::DECODE-WM-SIZE-HINTS XLIB:FONT-P XLIB:GCONTEXT-DASHES
            XLIB:IMAGE-X-P XLIB::DEALLOCATE-CONTEXT
            XLIB:DISPLAY-AFTER-FUNCTION XLIB:WINDOW-BIT-GRAVITY
            XLIB:CURSOR-P XLIB:DRAWABLE-X XLIB:DRAWABLE-Y
            XLIB:DISPLAY-FORCE-OUTPUT XLIB::INDEX-INCREMENT
            XLIB:FREE-COLORMAP XLIB:DISPLAY-PLIST
            XLIB:DISCARD-FONT-INFO XLIB::ALLOCATE-EXTENSION-EVENT-CODE
            XLIB:WINDOW-BACKING-PLANES XLIB:WINDOW-MAP-STATE
            XLIB:VISUAL-INFO-P XLIB:GCONTEXT-JOIN-STYLE
            XLIB:GCONTEXT-TS-X XLIB:IMAGE-Z-P XLIB:WINDOW-SAVE-UNDER
            XLIB::DISPLAY-LAST-FLUSHED-REQUEST XLIB::BUFFER-LOCK
            XLIB:CLOSE-FONT XLIB::BUFFER-TBUF16 XLIB:GCONTEXT-ARC-MODE
            XLIB::DISPLAY-OBUF8 XLIB:WINDOW-EVENT-MASK
            XLIB::DISPLAY-TBUF16 XLIB::BUFFER-WRITE-FUNCTION
            XLIB::DISPLAY-INPUT-STREAM
            XLIB::BUFFER-FORCE-OUTPUT-FUNCTION
            XLIB:WINDOW-ALL-EVENT-MASKS XLIB::DISPLAY-DEAD
            XLIB::BUFFER-CLOSE-FUNCTION XLIB:COLOR-P
            XLIB:WINDOW-OVERRIDE-REDIRECT XLIB::DISPLAY-FLUSH-INHIBIT
            XLIB:GCONTEXT-FOREGROUND XLIB::BUFFER-INPUT-FUNCTION
            XLIB::FORCE-GCONTEXT-CHANGES-INTERNAL XLIB:DRAWABLE-DEPTH
            XLIB:DISPLAY-DEFAULT-SCREEN
            XLIB::BUFFER-INPUT-WAIT-FUNCTION
            XLIB::DISPLAY-WRITE-FUNCTION XLIB::BUFFER-LISTEN-FUNCTION
            XLIB:WINDOW-COLORMAP XLIB:IMAGE-XY-P
            XLIB:GCONTEXT-CAP-STYLE XLIB::DISPLAY-FORCE-OUTPUT-FUNCTION
            XLIB::DISPLAY-CLOSE-FUNCTION
            XLIB:RESOURCE-DATABASE-TIMESTAMP
            XLIB::INCF-RESOURCE-DATABASE-TIMESTAMP
            XLIB::INITIALIZE-RESOURCE-DATABASE
            XLIB::DISPLAY-INPUT-FUNCTION
            XLIB::DISPLAY-INPUT-WAIT-FUNCTION XLIB:GCONTEXT-P
            XLIB::PENDING-COMMAND-PROCESS XLIB:WINDOW-CLASS
            XLIB::DISPLAY-LISTEN-FUNCTION XLIB:GCONTEXT-FUNCTION
            XLIB:COPY-COLORMAP-AND-FREE XLIB::OPEN-FONT-INTERNAL
            XLIB::IMAGE-XY-PLIST XLIB:DRAWABLE-WIDTH
            XLIB::DISPLAY-EVENT-LOCK XLIB::QUERY-FONT
            XLIB:WINDOW-CURSOR XLIB:ICON-SIZES
            XLIB::START-PENDING-COMMAND XLIB::DISPLAY-EVENT-QUEUE-LOCK
            XLIB::BUFFER-FORCE-OUTPUT XLIB:SCREEN-P
            XLIB:DRAWABLE-HEIGHT XLIB:DISPLAY-P XLIB::FONT-FONT-INFO
            XLIB::DEALLOCATE-REPLY-BUFFER XLIB:WINDOW-BACKING-PIXEL
            XLIB:DISPLAY-BYTE-ORDER XLIB::DISPLAY-FONT-CACHE
            XLIB:BITMAP-FORMAT-P XLIB::IMAGE-Z-PLIST XLIB:FONT-ID
            XLIB:PIXMAP-FORMAT-P XLIB::DISPLAY-RESOURCE-ID-BYTE
            XLIB:WM-ZOOM-HINTS XLIB::DISPLAY-CLOSE-DOWN-MODE
            XLIB:WINDOW-VISUAL XLIB:TRACE-DISPLAY
            XLIB:WINDOW-COLORMAP-INSTALLED-P
            XLIB:SUSPEND-DISPLAY-TRACING XLIB:RESUME-DISPLAY-TRACING
            XLIB:GCONTEXT-CLIP-X XLIB:UNTRACE-DISPLAY
            XLIB:WINDOW-GRAVITY XLIB:WINDOW-BACKING-STORE
            XLIB::RESOURCE-DATABASE-VALUE XLIB:WINDOW-VISUAL-INFO
            XLIB:DRAWABLE-ROOT XLIB::RATIONAL
            XLIB:WINDOW-DO-NOT-PROPAGATE-MASK)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM T) T) XLIB::READ-EVENT-INPUT
            XLIB::MAPPING-MATCHES-P XLIB::RESTORE-GCONTEXT-TEMP-STATE
            XLIB:DEFAULT-KEYSYM-TRANSLATE)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T T) FIXNUM) XLIB::SET-RGB-COLORMAPS
            XLIB:RECOLOR-CURSOR XLIB:DRAW-SEGMENTS
            XLIB:DEFAULT-KEYSYM-INDEX)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) STRING) XLIB:DISPLAY-VENDOR-NAME
            XLIB:DISPLAY-AUTHORIZATION-NAME
            XLIB:DISPLAY-AUTHORIZATION-DATA)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                (T T INTEGER FIXNUM FIXNUM FIXNUM FIXNUM T FIXNUM
                   FIXNUM)
                FIXNUM)
            XLIB:COPY-PLANE)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM FIXNUM) T)
            XLIB::CHANGE-DRAWABLE-GEOMETRY XLIB:KEYCODE->KEYSYM)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM FIXNUM T FIXNUM FIXNUM *) T)
            XLIB:WARP-POINTER-IF-INSIDE)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T T INTEGER T T) FIXNUM)
            XLIB:SET-STANDARD-COLORMAP)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T FIXNUM FIXNUM) FIXNUM) XLIB:REPARENT-WINDOW)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T T T) FIXNUM) XLIB::CHANGE-ACCESS-HOST)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM INTEGER) T) XLIB::STATE-KEYSYMP
            XLIB::CHANGE-WINDOW-ATTRIBUTE)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T FIXNUM FIXNUM FIXNUM FIXNUM T FIXNUM FIXNUM)
                FIXNUM)
            XLIB:COPY-AREA)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T) XLIB:STRINGABLE)
            XLIB::RESOURCE-DATABASE-NAME)) 
(PROCLAIM '(FTYPE (FUNCTION (T T FIXNUM) FIXNUM) XLIB::SET-ATOM-ID)) 
(PROCLAIM '(FTYPE (FUNCTION ((VECTOR T) T T) T) XLIB::ENCODE-MASK)) 
(PROCLAIM '(FTYPE (FUNCTION (STRING *) *) XLIB::PARSE-RESOURCE)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM T FIXNUM FIXNUM T) T)
            XLIB::WRITE-VECTOR-CHAR2B-WITH-TRANSFORM
            XLIB::WRITE-LIST-CARD8-WITH-TRANSFORM
            XLIB::WRITE-VECTOR-CARD8-WITH-TRANSFORM
            XLIB::WRITE-LIST-CARD16-WITH-TRANSFORM
            XLIB::WRITE-VECTOR-CARD16-WITH-TRANSFORM
            XLIB::WRITE-LIST-INT16-WITH-TRANSFORM
            XLIB::WRITE-VECTOR-INT16-WITH-TRANSFORM
            XLIB::WRITE-LIST-CARD32-WITH-TRANSFORM
            XLIB::WRITE-VECTOR-CARD32-WITH-TRANSFORM
            XLIB::WRITE-LIST-CHAR2B-WITH-TRANSFORM)) 
(PROCLAIM
    '(FTYPE (FUNCTION ((VECTOR UNSIGNED-CHAR) T FIXNUM FIXNUM) T)
            XLIB::BUFFER-WRITE-DEFAULT)) 
(PROCLAIM '(FTYPE (FUNCTION (T FIXNUM T T) T) XLIB::READ-ERROR-INPUT)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM (VECTOR UNSIGNED-CHAR) T FIXNUM FIXNUM)
                T)
            XLIB::READ-SIMPLE-ARRAY-CARD8-WITH-TRANSFORM)) 
(PROCLAIM
    '(FTYPE (FUNCTION
                (T FIXNUM (VECTOR UNSIGNED-SHORT) T FIXNUM FIXNUM) T)
            XLIB::READ-SIMPLE-ARRAY-CARD16-WITH-TRANSFORM)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM (VECTOR T) T FIXNUM FIXNUM) T)
            XLIB::READ-SIMPLE-ARRAY-CARD32-WITH-TRANSFORM)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM T T FIXNUM FIXNUM) T)
            XLIB::READ-LIST-CARD8-WITH-TRANSFORM
            XLIB::READ-VECTOR-CARD8-WITH-TRANSFORM
            XLIB::READ-LIST-CARD16-WITH-TRANSFORM
            XLIB::READ-VECTOR-CARD16-WITH-TRANSFORM
            XLIB::READ-LIST-CARD32-WITH-TRANSFORM
            XLIB::READ-VECTOR-CARD32-WITH-TRANSFORM)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM BIT-VECTOR) T)
            XLIB::WRITE-BITVECTOR256)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM T *) T) XLIB::WRITE-SEQUENCE-INT8
            XLIB::WRITE-SEQUENCE-CARD16 XLIB::WRITE-SEQUENCE-INT16
            XLIB::WRITE-SEQUENCE-INT32 XLIB::WRITE-SEQUENCE-CHAR2B
            XLIB::KEYSYM-SHIFT-P XLIB::WRITE-SEQUENCE-CARD8
            XLIB::WRITE-SEQUENCE-CARD32 XLIB::WRITE-SEQUENCE-CHAR)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T FIXNUM FIXNUM FIXNUM FIXNUM FIXNUM) T)
            XLIB::COPY-PIXARRAY)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T) *) XLIB:SELECTION-OWNER
            XLIB::STRINGABLE-EQUAL XLIB::ENQUEUE-EVENT
            XLIB::REPORT-ASYNCHRONOUS-ERRORS XLIB:GET-STANDARD-COLORMAP
            XLIB:ALLOC-COLOR XLIB:QUERY-EXTENSION XLIB:LOOKUP-COLOR)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T *) T) XLIB:GCONTEXT-FONT
            XLIB::PRINT-HISTORY-DESCRIPTION XLIB::SET-ROOT-RESOURCES
            XLIB::PRINT-DB XLIB::DISPLAY-CONNECT
            XLIB::SELECTION-REQUEST-EVENT-PUT
            XLIB::SELECTION-NOTIFY-EVENT-PUT
            XLIB::KEYMAP-NOTIFY-EVENT-PUT XLIB::BUFFER-CLOSE-DEFAULT
            XLIB::CLOSE-BUFFER XLIB::REQUEST-NAME XLIB::X-ERROR
            XLIB::HOST-ADDRESS XLIB::CHECK-CONSISTENCY
            XLIB:CHANGE-POINTER-CONTROL XLIB:ROOT-RESOURCES
            XLIB::GRAPHICS-EXPOSURE-EVENT-PUT
            XLIB::NO-EXPOSURE-EVENT-PUT
            XLIB::VISIBILITY-NOTIFY-EVENT-PUT
            XLIB::CREATE-NOTIFY-EVENT-PUT
            XLIB::DESTROY-NOTIFY-EVENT-PUT XLIB::UNMAP-NOTIFY-EVENT-PUT
            XLIB::MAP-NOTIFY-EVENT-PUT XLIB::MAP-REQUEST-EVENT-PUT
            XLIB:CHARACTER->KEYSYMS XLIB::REPARENT-NOTIFY-EVENT-PUT
            XLIB::CONFIGURE-NOTIFY-EVENT-PUT
            XLIB::CONFIGURE-REQUEST-EVENT-PUT
            XLIB::GRAVITY-NOTIFY-EVENT-PUT
            XLIB::RESIZE-REQUEST-EVENT-PUT
            XLIB::CIRCULATE-NOTIFY-EVENT-PUT
            XLIB::CIRCULATE-REQUEST-EVENT-PUT
            XLIB::PROPERTY-NOTIFY-EVENT-PUT
            XLIB::MOTION-NOTIFY-EVENT-PUT
            XLIB::SELECTION-CLEAR-EVENT-PUT XLIB:ROTATE-CUT-BUFFERS
            XLIB::ENTER-NOTIFY-EVENT-PUT XLIB::KEY-PRESS-EVENT-PUT
            XLIB::FOCUS-IN-EVENT-PUT XLIB::COLORMAP-NOTIFY-EVENT-PUT
            XLIB::MAPPING-NOTIFY-EVENT-PUT XLIB::EXPOSURE-EVENT-PUT
            XLIB:CHANGE-KEYBOARD-CONTROL XLIB:CLOSE-DISPLAY
            XLIB:LIST-PROPERTIES XLIB:CLEAR-AREA XLIB:BELL
            XLIB:OPEN-DISPLAY XLIB:SET-STANDARD-PROPERTIES
            XLIB:KEYBOARD-MAPPING XLIB:UNGRAB-POINTER
            XLIB:UNGRAB-KEYBOARD XLIB:SET-WM-PROPERTIES XLIB:SHOW-TRACE
            XLIB:DESCRIBE-TRACE XLIB:COPY-IMAGE)) 
(PROCLAIM '(FTYPE (FUNCTION (T CHARACTER) T) XLIB::CHAR-MEMQ)) 
(PROCLAIM
    '(FTYPE (FUNCTION (T T) T) XLIB:SET-CLOSE-DOWN-MODE XLIB:OPEN-FONT
            XLIB::SET-GCONTEXT-ARC-MODE XLIB::SET-GCONTEXT-DASHES
            XLIB:SET-POINTER-MAPPING XLIB::SET-GCONTEXT-FONT
            XLIB::SET-WINDOW-BACKGROUND XLIB::SET-WINDOW-BORDER
            XLIB::SET-WINDOW-BIT-GRAVITY XLIB::SET-WINDOW-GRAVITY
            XLIB::SET-WINDOW-BACKING-STORE
            XLIB::SET-WINDOW-BACKING-PLANES XLIB:FIND-ATOM
            XLIB::SET-WINDOW-BACKING-PIXEL XLIB::SET-WINDOW-SAVE-UNDER
            XLIB::SET-WINDOW-OVERRIDE-REDIRECT
            XLIB::SET-WINDOW-COLORMAP XLIB::SET-WINDOW-CURSOR
            XLIB::SET-DRAWABLE-X XLIB::SET-DRAWABLE-Y
            XLIB::SET-DRAWABLE-WIDTH XLIB::SET-DRAWABLE-HEIGHT
            XLIB::SET-DRAWABLE-BORDER-WIDTH XLIB::SET-WM-HINTS
            XLIB::DELETE-RESOURCE-INTERNAL
            XLIB::DEALLOCATE-RESOURCE-ID-INTERNAL
            XLIB::SET-WM-NORMAL-HINTS XLIB::SET-WM-ZOOM-HINTS
            XLIB::PRINT-DISPLAY-NAME XLIB::SET-ICON-SIZES
            XLIB::SET-WM-PROTOCOLS XLIB::SET-WM-COLORMAP-WINDOWS
            XLIB::SET-TRANSIENT-FOR XLIB::NOTE-INPUT-COMPLETE
            XLIB::BUFFER-INPUT-WAIT XLIB::SET-FONT-PATH
            XLIB::DEQUEUE-EVENT XLIB:DELETE-RESOURCE
            XLIB::BUFFER-INPUT-WAIT-DEFAULT XLIB::REPORT-REQUEST-ERROR
            XLIB::DECODE-RESOURCE-ERROR XLIB::MAKE-INDEX-OP
            XLIB::SET-WM-COMMAND XLIB::TRACE-EVENT-PRINT
            XLIB::SET-GCONTEXT-LINE-STYLE XLIB::SET-GCONTEXT-CAP-STYLE
            XLIB::SET-GCONTEXT-JOIN-STYLE XLIB::SET-GCONTEXT-FILL-STYLE
            XLIB::SET-GCONTEXT-FILL-RULE XLIB::SET-GCONTEXT-TILE
            XLIB::SET-GCONTEXT-STIPPLE XLIB:EVENT-HANDLER
            XLIB:GET-EXTERNAL-EVENT-CODE XLIB:RGB-COLORMAPS
            XLIB:INTERN-ATOM XLIB::READ-REPLY
            XLIB::STOP-PENDING-COMMAND
            XLIB::SET-GCONTEXT-SUBWINDOW-MODE
            XLIB::SET-GCONTEXT-EXPOSURES XLIB:FONT-PROPERTY
            XLIB:SET-ACCESS-CONTROL XLIB:MERGE-RESOURCES XLIB:ATOM-NAME
            XLIB:COPY-GCONTEXT XLIB:DESCRIBE-REQUEST
            XLIB:DESCRIBE-REPLY XLIB:DESCRIBE-ERROR)) 
(PROCLAIM
    '(FTYPE (FUNCTION (INTEGER (VECTOR UNSIGNED-CHAR) FIXNUM) T)
            XLIB::ASET-CARD32)) 